/*****************************************************************
 * Pic 16F84A to Velleman K8000 demonstration program
 *
 * Copyright (c) 2012 - Vanderhaegen Bart
 *
 * This code demonstrates how you can control the Velleman K8000
 * with a PIC 16F84A microcontroller. This particular controller
 * doesn't support native I2C so I had to write my own I2C 
 * functions.
 *
 ****************************************************************/


#include <htc.h>;


// declaration of the functions
void delay(void);
void i2cStart(void);
void i2cStop(void);
bit waitForAck(void);
void clock(void);
void sendData(char Data);
void giveI2Ccommand(char address,char data);
void delay_ms(int ms);
void sendAck(void);
void sendEffect(char chip1,char chip2);

// configuration word of the PIC microcontroller
__CONFIG(0x3FF9);


// define I2C communications
#define _XTAL_FREQ 4000000
#define SCL TRISA0  // serial clock (requires pull-up)
#define SDA TRISA1  // serial data(requires pull-up
#define SCL_IN  RA0 //
#define SDA_IN  RA1	//




void main(void) {

	// set state of the output registers
	TRISA=0x00;
	TRISB=0x00;
	PORTB=0x00;

	// set initial state of the I2C bus
	SCL_IN=0;
	SDA_IN=0;
	SCL=1;
	SDA=1;



	while (1) {	
		sendEffect(0b00000000,0b00000001);
		sendEffect(0b00000000,0b00000010);
		sendEffect(0b00000000,0b00000100);
		sendEffect(0b00000000,0b00001000);
		sendEffect(0b00000000,0b00010000);
		sendEffect(0b00000000,0b00100000);
		sendEffect(0b00000000,0b01000000);
		sendEffect(0b00000000,0b10000000);
		sendEffect(0b00000001,0b00000000);
		sendEffect(0b00000010,0b00000000);
		sendEffect(0b00000100,0b00000000);
		sendEffect(0b00001000,0b00000000);
		sendEffect(0b00010000,0b00000000);
		sendEffect(0b00100000,0b00000000);
		sendEffect(0b01000000,0b00000000);
		sendEffect(0b10000000,0b00000000);
	}
}



/***************************************************************************************
 * K8000 handling
 * This function is hardcoded to use the I2C chips PCF8574AP with hardware addresses
 * set to 0x70 and 0x72. They correspond to a K8000 card that is set to card 0.
 **************************************************************************************/
	void sendEffect(char chip1,char chip2) {
		giveI2Ccommand(0x72,~chip1);
		giveI2Ccommand(0x70,~chip2);
		__delay_ms(100);
	}

	



/***************************************************************************************
 * Implementation of the I2C protocol
 * Note: I wrote this implementation myself. This code still needs tuning. In some cases
 * a communication error will lock up the PIC controller.
 **************************************************************************************/
// send 8 bits of data over the I2C bus
  void sendData(char Data) {
    unsigned char i;
  
    for (i=8;i>0;i--) {
      if(Data & 0x80) {
        SDA=1;
      }
      else {
        SDA=0;
      }

      SCL=1;
      delay();
      Data<<=1;
      SCL=0;
      delay();
    }
  }


  // send the I2C start sequence
  // SDA must go from high to low while SCL is high.
  void i2cStart() {
    SDA=1;    
    delay();
    SCL=1;    
    delay();  
    SDA=0;       
    delay();
    SCL=0; 
    delay();
  }

  
  // send the I2C stop sequence
  // SDA must go from low to high while SCL is high
  void i2cStop() {
    SDA=0;    
    delay();  
    SCL=1;    
    delay();    
    SDA=1;    
    delay();
  }



  // wait for an acknowledge signal
  // Receiver must make SDA low while SCL is high
  // The waitForAck times out after approximately 10 ms
  bit waitForAck() {
    SDA=1;      
    delay();    
    SCL=1;  
    unsigned int timeout=1923;
    while (SDA_IN && (--timeout > 0));
    SCL=0;    
    delay();
    return (timeout>0);
  }



  // send the clock pulse
  // SCL must be made high for a short period of time
  void clock() {
    delay();  
    SCL=1;    
    delay();    
    SCL=0;    
    delay();  
  }


  // send the acknowledge pulse required by the slave device
  void sendAck() {
    SDA_IN=0; 
    SDA=0;    
    delay();
    SCL=1;      
    delay();    
    SCL=0;  
    delay();    
  }


  // delay execution
  void delay(void){
    __delay_us(25);
  }


  	// initiate a new I2C command
	void giveI2Ccommand(char address,char data) {
		i2cStart();	 
		sendData(address);
		waitForAck();
		sendData(data);
		waitForAck();
		i2cStop();
	}


